@@ -4,13 +4,22 @@ package IkiWiki::Plugin::orphans;
 
 use warnings;
 use strict;
-use IkiWiki 2.00;
+use IkiWiki 3.00;
 
-sub import { #{{{
+sub import {
+	hook(type => "getsetup", id => "orphans", call => \&getsetup);
 	hook(type => "preprocess", id => "orphans", call => \&preprocess);
-} # }}}
+}
 
-sub preprocess (@) { #{{{
+sub getsetup () {
+	return 
+		plugin => {
+			safe => 1,
+			rebuild => undef,
+		},
+}
+
+sub preprocess (@) {
 	my %params=@_;
 	$params{pages}="*" unless defined $params{pages};
 	
@@ -34,7 +43,7 @@ sub preprocess (@) { #{{{
 		next if grep { 
 			length $_ &&
 			($_ !~ /\/\Q$discussion\E$/i || ! $config{discussion}) &&
-			bestlink($page, $_) !~ /^($page|)$/ 
+			bestlink($page, $_) !~ /^(\Q$page\E|)$/ 
 		} @{$links{$page}};
 		push @orphans, $page;
 	}
@@ -49,6 +58,6 @@ sub preprocess (@) { #{{{
 				"</li>"
 			} sort @orphans).
 		"</ul>\n";
-} # }}}
+}
 
 1
